"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.subscribeToEditor = exports.createEditorListener = void 0;
var _isBrowser = require("../functions/is-browser");
var _isFromTrustedHost = require("../functions/is-from-trusted-host");
var _initEditing = require("../scripts/init-editing");
var _logger = require("./logger");
const createEditorListener = ({
  model,
  trustedHosts,
  callbacks
}) => {
  return event => {
    if (!(0, _isFromTrustedHost.isFromTrustedHost)(trustedHosts, event)) {
      return;
    }
    const {
      data
    } = event;
    if (data) {
      switch (data.type) {
        case 'builder.configureSdk':
          {
            callbacks.configureSdk(data.data);
            break;
          }
        case 'builder.triggerAnimation':
          {
            callbacks.animation(data.data);
            break;
          }
        case 'builder.resetState':
          {
            const messageContent = data.data;
            const modelName = messageContent.model;
            const newState = messageContent === null || messageContent === void 0 ? void 0 : messageContent.state;
            if (modelName === model && newState) {
              callbacks.stateUpdate(newState);
            }
            break;
          }
        case 'builder.contentUpdate':
          {
            const messageContent = data.data;
            const key = messageContent.key || messageContent.alias || messageContent.entry || messageContent.modelName;
            const contentData = messageContent.data;
            if (key === model) {
              callbacks.contentUpdate(contentData);
            }
            break;
          }
      }
    }
  };
};
exports.createEditorListener = createEditorListener;
/**
 * Subscribes to the Builder editor and listens to `content` updates of a certain `model`.
 * Sends the updated `content` to the `callback` function.
 */
const subscribeToEditor = ({
  model,
  apiKey,
  callback,
  trustedHosts
}) => {
  if (!_isBrowser.isBrowser) {
    _logger.logger.warn('`subscribeToEditor` only works in the browser. It currently seems to be running on the server.');
    return () => {};
  }
  (0, _initEditing.setupBrowserForEditing)({
    modelName: model,
    apiKey
  });
  const listener = createEditorListener({
    callbacks: {
      contentUpdate: callback,
      animation: () => {},
      configureSdk: () => {},
      stateUpdate: () => {}
    },
    model,
    trustedHosts
  });
  window.addEventListener('message', listener);
  return () => {
    window.removeEventListener('message', listener);
  };
};
exports.subscribeToEditor = subscribeToEditor;
//# sourceMappingURL=subscribe-to-editor.js.map